""" DeskNotes Application

	Copyright (c) 2000 Colin Stewart (http://www.owlfish.com/)
	All rights reserved.
		
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.
	3. The name of the author may not be used to endorse or promote products
	   derived from this software without specific prior written permission.
	
	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	
	If you make any bug fixes or feature enhancements please let me know!

"""
// File: $RCSFile$
// Revision: $Revision: 1.3 $
// Date: $Date: 2000/03/26 16:58:17 $

#include "DeskNoteApp.h"

main () {
	DeskNoteApp *myApp;
	myApp = new DeskNoteApp ();
	myApp -> Run ();
	delete myApp;
}

const char *app_signature = "application/x-vnd.cms103-DeskNotes";

DeskNoteApp::DeskNoteApp ():BApplication (app_signature) {
	BRect rect, scrRect;
	BPath dir;
	BFile *file;
	BMessage *msg;
	BScreen *currentScreen;
	bool SettingsOK = true;
	int fileVer;
	char *buffer = new char [1024];
	
	find_directory (B_USER_SETTINGS_DIRECTORY, &dir);
	dir.Append ("DeskNotes Settings");
	file = new BFile (dir.Path(), B_READ_ONLY);
	msg = new BMessage ();
	if (file -> InitCheck () == B_OK) {
		if (file -> Read (buffer, strlen (header)) != (ssize_t)strlen (header)) SettingsOK = false;
		if (strcasecmp (header, buffer) != 0) SettingsOK = false;
		file -> Read (&fileVer, sizeof (NotesVersion));
		if (fileVer != NotesVersion) SettingsOK = false;
		msg -> Unflatten (file);
	}
	else {
		SettingsOK = false;
	}
	delete file;
	delete buffer;
	if (SettingsOK) {
		currentScreen = new BScreen();			// This locks the screen.
		scrRect = currentScreen -> Frame();		// Find out how big the screen is.
		delete currentScreen;					// Delete the object, unlock the screen.
		msg -> FindRect ("windowPos", &rect);
		if (rect.right > scrRect.right) {
			// If the window would be off-screen then try correcting it...
			float width, scrWidth;
			scrWidth = scrRect.right - scrRect.left;
			width = rect.right - rect.left;
			if (width > scrWidth) width = scrWidth - 20;
			rect.right = scrRect.right - 2;
			rect.left = rect.right - width;	
		}
		if (rect.bottom > scrRect.bottom) {
			// If the window would be off-screen then try correcting it...
			float height, scrHeight;
			scrHeight = scrRect.bottom - scrRect.top;
			height = rect.bottom - rect.top;
			if (height > scrHeight) height = scrHeight - 20;
			rect.bottom = scrRect.bottom - 2;
			rect.top = rect.bottom - height;
		}
	}
	else {
		rect.Set (100,80,240,160);				// Inital size for the window.
	}
	
	myNote = new DeskNoteWindow (rect);
	myNote -> RestoreNote (msg);
	myNote -> Show ();
	delete msg;
}

bool DeskNoteApp::QuitRequested () {
	BAlert *lert;
	BPath dir;
	BFile *file;
	BMessage *msg;
	int ver = NotesVersion;
	
	if (find_directory (B_USER_SETTINGS_DIRECTORY, &dir) == B_OK) {
		// This is where we save our notes position, size, and text content.
		dir.Append ("DeskNotes Settings");
		file = new BFile (dir.Path() , B_READ_WRITE | B_CREATE_FILE);
		file -> Write (header, strlen (header));
		file -> Write (&ver, sizeof (ver));
		msg = getSettings();
		msg -> Flatten (file);
		delete file;
		delete msg;
	}
	else {
		lert = new BAlert ("DeskNotes", "Unable To Save Note!", "OK");
		lert -> Go();
	}
		
	return true;
}

BMessage * DeskNoteApp::getSettings () {
	BMessage *msg = new BMessage ();
	BRect rct;
	
	rct = myNote -> Frame();
	msg -> AddRect ("windowPos", rct);
	myNote -> SaveNote (msg);
	
	return msg;
}
	
	